@@ -20,17 +20,18 @@ from ape.exceptions import Abort, ApeException, ConversionError
 from ape.logging import LogLevel
 from ape.types import AddressType
 
-from silverback._click_ext import (
+from ._click_ext import (
     SectionedHelpGroup,
     auth_required,
     bot_path_callback,
     cls_import_callback,
     cluster_client,
-    display_login_message,
+    parse_globbed_arg,
     platform_client,
     timedelta_callback,
     token_amount_callback,
 )
+from .exceptions import ClientError
 
 if TYPE_CHECKING:
     from ape.api import AccountAPI, EcosystemAPI, NetworkAPI, ProviderAPI
@@ -103,8 +104,9 @@ def _network_callback(ctx, param, val):
     callback=cls_import_callback,
 )
 @click.option("-x", "--max-exceptions", type=int, default=3)
+@click.option("--debug", is_flag=True, default=False)
 @click.argument("bot", required=False, callback=bot_path_callback)
-def run(cli_ctx, account, runner_class, recorder_class, max_exceptions, bot):
+def run(cli_ctx, account, runner_class, recorder_class, max_exceptions, debug, bot):
     """Run Silverback bot"""
     from silverback.runner import PollingRunner, WebsocketRunner
 
@@ -125,36 +127,84 @@ def run(cli_ctx, account, runner_class, recorder_class, max_exceptions, bot):
         recorder=recorder_class() if recorder_class else None,
         max_exceptions=max_exceptions,
     )
-    asyncio.run(runner.run())
+    asyncio.run(runner.run(), debug=debug)
 
 
 @cli.command(section="Local Commands")
-@click.option("--generate", is_flag=True, default=False)
-@click.argument("path", required=False, type=str, default="bots")
-def build(generate, path):
-    """Generate Dockerfiles and build bot images"""
-    from silverback._build_utils import build_docker_images, generate_dockerfiles
+@click.option(
+    "-g",
+    "--generate",
+    is_flag=True,
+    default=False,
+    help="Generate Dockerfiles first. Defaults to false.",
+)
+@click.option(
+    "-t",
+    "--tag-base",
+    default=None,
+    help=(
+        "The base to use to tag the image. "
+        "The bot name (or 'bot') is appended to it, following a '-' separator. "
+        "Defaults to using the name of the folder you are building from."
+    ),
+)
+@click.option(
+    "--version",
+    default="latest",
+    metavar="VERSION",
+    help="Version to use in tag. Defaults to 'latest'.",
+)
+@click.option(
+    "--push",
+    is_flag=True,
+    default=False,
+    help="Push image to logged-in registry. Defaults to false.",
+)
+@click.argument("path", required=False, default=None)
+def build(generate, tag_base, version, push, path):
+    """
+    Generate Dockerfiles and build bot container images
+
+    When '--tag-base' is used, you can control the base of the tag for the image.
+    For example, '--tag-base project' with bots 'botA.py' and 'botB.py' (under 'bots/') produces
+    '-t project-bota:latest' and '-t project-botb:latest' respectively.
+
+    For building to push to a specific image registry, use '--tag-base' to correctly tag images.
+    Using '--tag-base ghcr.io/myorg/myproject' with the previous example
+    '-t ghcr.io/myorg/project-bota:latest' and '-t ghcr.io/myorg/project-botb:latest' respectively.
+    """
+    from silverback._build_utils import (
+        IMAGES_FOLDER_NAME,
+        build_docker_images,
+        generate_dockerfiles,
+    )
 
     if generate:
         if (
-            not (path := Path.cwd() / path).exists()
+            path is not None
+            and not (path := Path.cwd() / f"{path}.py").exists()
+            and not (path := Path.cwd() / path).exists()
+        ) or (
+            path is None
+            and not (path := Path.cwd() / "bots").exists()
             and not (path := Path.cwd() / "bot").exists()
             and not (path := Path.cwd() / "bot.py").exists()
         ):
-            raise FileNotFoundError(
-                f"The bots directory '{path}', 'bot/' and 'bot.py' does not exist in your path. "
-                f"You should have a '{path}/' or 'bot/' folder, or a 'bot.py' file in the root "
-                "of your project."
+            raise click.ClickException(
+                f"The path '{path.relative_to(Path.cwd())}' does not exist in project. "
+                "This command can auto-detect 'bot.py' or 'bot/' folder in project root"
+                ", or process all '*.py' bots in  'bots/' folder."
             )
+
         generate_dockerfiles(path)
 
-    if not (path := Path.cwd() / ".silverback-images").exists():
-        raise FileNotFoundError(
-            f"The dockerfile directory '{path}' does not exist. "
-            "You should have a `{path}/` folder in the root of your project."
+    if not (Path.cwd() / IMAGES_FOLDER_NAME).exists():
+        raise click.ClickException(
+            f"The dockerfile cache folder '{IMAGES_FOLDER_NAME}' does not exist. "
+            "You can run `silverback build --generate` to generate it and build."
         )
 
-    build_docker_images(path)
+    build_docker_images(tag_base=tag_base, version=version, push=push)
 
 
 @cli.command(cls=ConnectedProviderCommand, section="Local Commands")
@@ -167,12 +217,16 @@ def build(generate, path):
 @click.option("-w", "--workers", type=int, default=2)
 @click.option("-x", "--max-exceptions", type=int, default=3)
 @click.option("-s", "--shutdown_timeout", type=int, default=90)
+@click.option("--debug", is_flag=True, default=False)
 @click.argument("bot", required=False, callback=bot_path_callback)
-def worker(cli_ctx, account, workers, max_exceptions, shutdown_timeout, bot):
+def worker(cli_ctx, account, workers, max_exceptions, shutdown_timeout, debug, bot):
     """Run Silverback task workers (advanced)"""
     from silverback.worker import run_worker
 
-    asyncio.run(run_worker(bot.broker, worker_count=workers, shutdown_timeout=shutdown_timeout))
+    asyncio.run(
+        run_worker(bot.broker, worker_count=workers, shutdown_timeout=shutdown_timeout),
+        debug=debug,
+    )
 
 
 @cli.command(section="Cloud Commands (https://silverback.apeworx.io)")
@@ -181,7 +235,14 @@ def login(auth: "FiefAuth"):
     """Login to ApeWorX Authorization Service (https://account.apeworx.io)"""
 
     auth.authorize()
-    display_login_message(auth, auth.client.base_url)
+    userinfo = auth.current_user()
+    user_id = userinfo["sub"]
+    username = userinfo["fields"].get("username")
+    click.echo(
+        f"{click.style('INFO', fg='blue')}: "
+        f"Logged in to '{click.style(auth.client.base_url, bold=True)}' as "
+        f"'{click.style(username if username else user_id, bold=True)}'"
+    )
 
 
 @cli.group(cls=SectionedHelpGroup, section="Cloud Commands (https://silverback.apeworx.io)")
@@ -198,7 +259,7 @@ def workspaces():
 
 
 @workspaces.command(name="list", section="Platform Commands (https://silverback.apeworx.io)")
-@platform_client
+@platform_client()
 def list_workspaces(platform: "PlatformClient"):
     """List available workspaces for your account"""
 
@@ -216,7 +277,7 @@ def list_workspaces(platform: "PlatformClient"):
 
 @workspaces.command(name="info", section="Platform Commands (https://silverback.apeworx.io)")
 @click.argument("workspace")
-@platform_client
+@platform_client()
 def workspace_info(platform: "PlatformClient", workspace: str):
     """Get Configuration information about a WORKSPACE"""
 
@@ -244,7 +305,7 @@ def workspace_info(platform: "PlatformClient", workspace: str):
     "workspace_slug",
     help="Slug for new workspace",
 )
-@platform_client
+@platform_client()
 def new_workspace(
     platform: "PlatformClient",
     workspace_name: str | None,
@@ -286,7 +347,7 @@ def new_workspace(
     help="Update slug for workspace",
 )
 @click.argument("workspace")
-@platform_client
+@platform_client()
 def update_workspace(
     platform: "PlatformClient",
     workspace: str,
@@ -314,7 +375,7 @@ def update_workspace(
 
 @workspaces.command(name="delete", section="Platform Commands (https://silverback.apeworx.io)")
 @click.argument("workspace")
-@platform_client
+@platform_client()
 def delete_workspace(platform: "PlatformClient", workspace: str):
     """Delete an empty Workspace on the Silverback Platform"""
 
@@ -330,7 +391,7 @@ def delete_workspace(platform: "PlatformClient", workspace: str):
 
 @cluster.command(name="list", section="Platform Commands (https://silverback.apeworx.io)")
 @click.argument("workspace")
-@platform_client
+@platform_client()
 def list_clusters(platform: "PlatformClient", workspace: str):
     """List available clusters in a WORKSPACE"""
 
@@ -359,7 +420,7 @@ def list_clusters(platform: "PlatformClient", workspace: str):
     help="Slug for new cluster (Defaults to `name.lower()`)",
 )
 @click.argument("workspace")
-@platform_client
+@platform_client()
 def new_cluster(
     platform: "PlatformClient",
     workspace: str,
@@ -412,7 +473,7 @@ def new_cluster(
     help="Update slug for cluster",
 )
 @click.argument("cluster_path")
-@platform_client
+@platform_client()
 def update_cluster(
     platform: "PlatformClient",
     cluster_path: str,
@@ -448,6 +509,47 @@ def update_cluster(
     click.echo(f"{click.style('SUCCESS', fg='green')}: Updated '{updated_cluster.name}'")
 
 
+@cluster.command(name="migrate", section="Platform Commands (https://silverback.apeworx.io)")
+@click.option("--version", default=None)
+@click.argument("cluster_path")
+@platform_client()
+def migrate_cluster(
+    platform: "PlatformClient",
+    cluster_path: str,
+    version: str | None,
+):
+    """Migrate CLUSTER running software version to VERSION"""
+    if "/" not in cluster_path or len(cluster_path.split("/")) > 2:
+        raise click.BadArgumentUsage(f"Invalid cluster path: '{cluster_path}'")
+
+    workspace_name, cluster_name = cluster_path.split("/")
+    if not (workspace_client := platform.workspaces.get(workspace_name)):
+        raise click.BadArgumentUsage(f"Unknown workspace: '{workspace_name}'")
+
+    elif not (cluster := workspace_client.clusters.get(cluster_name)):
+        raise click.BadArgumentUsage(
+            f"Unknown cluster in workspace '{workspace_name}': '{cluster_name}'"
+        )
+
+    elif version and version not in (available_versions := workspace_client.available_versions):
+        available_versions_str = "', '".join(available_versions)
+        raise click.BadOptionUsage(
+            "version",
+            f"Cannot migrate to version '{version}', must be one of: '{available_versions_str}'",
+        )
+
+    click.echo(
+        f"{click.style('INFO', fg='blue')}: "
+        f"Migrating '{cluster_path}' from '{cluster.version}' to '{version or 'stable'}'"
+    )
+    workspace_client.migrate_cluster(str(cluster.id), version=version)
+    click.echo(f"{click.style('SUCCESS', fg='green')}: Migration of '{cluster.name}' started")
+    click.echo(
+        f"{click.style('INFO', fg='blue')}: "
+        "This may take a couple of minutes, check `silverback cluster info` for version change"
+    )
+
+
 @cluster.group(cls=SectionedHelpGroup, section="Platform Commands (https://silverback.apeworx.io)")
 def pay():
     """Pay for CLUSTER with Crypto using ApePay streaming payments"""
@@ -505,7 +607,7 @@ def _tier_choices():
     default=None,
     help="Time to fund stream for",
 )
-@platform_client
+@platform_client()
 def create_payment_stream(
     platform: "PlatformClient",
     network: "NetworkAPI",
@@ -626,7 +728,7 @@ def create_payment_stream(
 
     click.echo(
         f"{click.style('WARNING', fg='yellow')}: Cluster may take up to 1 hour to deploy."
-        " Check back in 10-15 minutes using `silverback cluster info` to start using your cluster."
+        " Check back in 2-5 minutes using `silverback cluster info` to start using your cluster."
     )
 
 
@@ -649,7 +751,7 @@ def create_payment_stream(
     default=None,
     help="Time to fund stream for",
 )
-@platform_client
+@platform_client()
 def fund_payment_stream(
     platform: "PlatformClient",
     network: "NetworkAPI",
@@ -722,7 +824,7 @@ def fund_payment_stream(
 @pay.command(name="cancel", cls=ConnectedProviderCommand)
 @account_option()
 @click.argument("cluster_path", metavar="CLUSTER")
-@platform_client
+@platform_client()
 def cancel_payment_stream(
     platform: "PlatformClient",
     network: "NetworkAPI",
@@ -758,7 +860,7 @@ def cancel_payment_stream(
 
 
 @cluster.command(name="info")
-@cluster_client
+@cluster_client()
 def cluster_info(cluster: "ClusterClient"):
     """Get configuration information about a CLUSTER"""
 
@@ -767,7 +869,7 @@ def cluster_info(cluster: "ClusterClient"):
 
 
 @cluster.command(name="health")
-@cluster_client
+@cluster_client()
 def cluster_health(cluster: "ClusterClient"):
     """Get health information about a CLUSTER"""
 
@@ -780,7 +882,7 @@ def registry():
 
 
 @registry.command(name="list")
-@cluster_client
+@cluster_client()
 def credentials_list(cluster: "ClusterClient"):
     """List container registry credentials in CLUSTER"""
 
@@ -793,7 +895,7 @@ def credentials_list(cluster: "ClusterClient"):
 
 @registry.command(name="info")
 @click.argument("name")
-@cluster_client
+@cluster_client()
 def credentials_info(cluster: "ClusterClient", name: str):
     """Show info about credential NAME in CLUSTER's registry"""
 
@@ -806,7 +908,7 @@ def credentials_info(cluster: "ClusterClient", name: str):
 @registry.command(name="new")
 @click.argument("name")
 @click.argument("registry")
-@cluster_client
+@cluster_client()
 def credentials_new(cluster: "ClusterClient", name: str, registry: str):
     """Add registry access credential NAME to CLUSTER's registry.
 
@@ -826,7 +928,7 @@ def credentials_new(cluster: "ClusterClient", name: str, registry: str):
 @registry.command(name="update")
 @click.argument("name")
 @click.option("-r", "--registry")
-@cluster_client
+@cluster_client()
 def credentials_update(cluster: "ClusterClient", name: str, registry: str | None = None):
     """Update credential NAME in CLUSTER's registry
 
@@ -845,7 +947,7 @@ def credentials_update(cluster: "ClusterClient", name: str, registry: str | None
 
 @registry.command(name="remove")
 @click.argument("name")
-@cluster_client
+@cluster_client()
 def credentials_remove(cluster: "ClusterClient", name: str):
     """Remove credential NAME from CLUSTER's registry"""
     if not (creds := cluster.credentials.get(name)):
@@ -862,7 +964,16 @@ def vars():
 
 def parse_envvars(ctx, name, value: list[str]) -> dict[str, str]:
     def parse_envar(item: str):
-        if not ("=" in item and len(item.split("=")) == 2):
+
+        if "=" not in item:
+            if not (envvar := os.environ.get(item)):
+                raise click.UsageError(
+                    f"Environment variable '{item}' has no value in your environment"
+                )
+
+            return item, envvar
+
+        elif len(item.split("=")) != 2:
             raise click.UsageError(f"Value '{item}' must be in form `NAME=VAL`")
 
         return item.split("=")
@@ -877,12 +988,12 @@ def parse_envvars(ctx, name, value: list[str]) -> dict[str, str]:
     "variables",
     multiple=True,
     type=str,
-    metavar="NAME=VAL",
+    metavar="NAME[=VAL]",
     callback=parse_envvars,
-    help="Environment variable key and value to add (Multiple allowed)",
+    help="Environment variable name or key and value to add (Multiple allowed)",
 )
 @click.argument("name")
-@cluster_client
+@cluster_client()
 def new_vargroup(cluster: "ClusterClient", variables: dict, name: str):
     """Create a new group of environment variables in a CLUSTER"""
 
@@ -894,7 +1005,7 @@ def new_vargroup(cluster: "ClusterClient", variables: dict, name: str):
 
 
 @vars.command(name="list")
-@cluster_client
+@cluster_client()
 def list_vargroups(cluster: "ClusterClient"):
     """List latest revisions of all variable groups in a CLUSTER"""
 
@@ -907,9 +1018,9 @@ def list_vargroups(cluster: "ClusterClient"):
 
 @vars.command(name="info")
 @click.argument("name")
-@cluster_client
+@cluster_client()
 def vargroup_info(cluster: "ClusterClient", name: str):
-    """Show latest revision of a variable GROUP in a CLUSTER"""
+    """Show information about a variable GROUP in a CLUSTER"""
 
     if not (vg := cluster.variable_groups.get(name)):
         raise click.UsageError(f"Unknown Variable Group '{name}'")
@@ -924,9 +1035,9 @@ def vargroup_info(cluster: "ClusterClient", name: str):
     "updated_vars",
     multiple=True,
     type=str,
-    metavar="NAME=VAL",
+    metavar="NAME[=VAL]",
     callback=parse_envvars,
-    help="Environment variable key and value to add/update (Multiple allowed)",
+    help="Environment variable name or key and value to add/update (Multiple allowed)",
 )
 @click.option(
     "-d",
@@ -938,7 +1049,7 @@ def vargroup_info(cluster: "ClusterClient", name: str):
     help="Environment variable name to delete (Multiple allowed)",
 )
 @click.argument("name")
-@cluster_client
+@cluster_client()
 def update_vargroup(
     cluster: "ClusterClient",
     name: str,
@@ -973,7 +1084,7 @@ def update_vargroup(
 
 @vars.command(name="remove")
 @click.argument("name")
-@cluster_client
+@cluster_client()
 def remove_vargroup(cluster: "ClusterClient", name: str):
     """
     Remove a variable GROUP from a CLUSTER
@@ -1003,7 +1114,7 @@ def bots():
     help="registry credentials to use to pull the image",
 )
 @click.argument("name")
-@cluster_client
+@cluster_client()
 def new_bot(
     cluster: "ClusterClient",
     image: str,
@@ -1020,6 +1131,9 @@ def new_bot(
     if name in cluster.bots:
         raise click.UsageError(f"Cannot use name '{name}' to create bot")
 
+    elif cluster.configuration and len(cluster.bots) >= cluster.configuration.bots:
+        click.secho("Number of bots in cluster is at paid limit", bold=True, fg="yellow")
+
     # NOTE: Check if credentials exist
     if credential_name is not None and credential_name not in cluster.credentials:
         raise click.UsageError(f"Unknown registry credentials '{credential_name}'")
@@ -1060,9 +1174,9 @@ def new_bot(
 
 
 @bots.command(name="list", section="Configuration Commands")
-@cluster_client
+@cluster_client()
 def list_bots(cluster: "ClusterClient"):
-    """List all bots in a CLUSTER (Regardless of status)"""
+    """List all bots in a CLUSTER by network (Regardless of status)"""
 
     if bots := list(cluster.bots.values()):
         groups: dict[str, dict[str, list["Bot"]]] = defaultdict(lambda: defaultdict(list))
@@ -1081,35 +1195,33 @@ def list_bots(cluster: "ClusterClient"):
 
 
 @bots.command(name="info", section="Configuration Commands")
-@click.argument("bot_name", metavar="BOT")
-@cluster_client
-def bot_info(cluster: "ClusterClient", bot_name: str):
-    """Get configuration information of a BOT in a CLUSTER"""
+@click.argument("name", metavar="BOT", default="*")
+@cluster_client()
+def bot_info(cluster: "ClusterClient", name: str):
+    """Get configuration information of one or more BOT(s) in a CLUSTER"""
 
-    if not (bot := cluster.bots.get(bot_name)):
-        raise click.UsageError(f"Unknown bot '{bot_name}'.")
+    for bot in parse_globbed_arg(name, cluster.bots):
+        bot_dump = bot.model_dump(
+            exclude={
+                "id",  # not needed
+                "name",  # key
+                # TODO: Remove from model
+                "credential_name",
+                # Must render later on
+                "environment",
+                # Display condensed version instead
+                "ecosystem",
+                "network",
+                "provider",
+            }
+        )
+        bot_dump["network"] = f"{bot.ecosystem}:{bot.network}:{bot.provider}"
+        if bot.environment:
+            bot_dump["environment"] = [
+                var.model_dump(exclude={"id", "created"}) for var in bot.vargroups
+            ]
 
-    # NOTE: Skip machine `.id`, and we already know it is `.name`
-    bot_dump = bot.model_dump(
-        exclude={
-            "id",
-            "name",
-            "credential_name",
-            "environment",
-            "ecosystem",
-            "network",
-            "provider",
-        }
-    )
-    bot_dump["network"] = f"{bot.ecosystem}:{bot.network}:{bot.provider}"
-
-    if bot.credential:
-        bot_dump["credential"] = bot.credential.model_dump(exclude={"id", "name"})
-
-    click.echo(yaml.safe_dump(bot_dump))
-    if bot.environment:
-        click.echo("environment:")
-        click.echo(yaml.safe_dump([var.model_dump() for var in bot.vargroups]))
+        click.echo(yaml.safe_dump({bot.name: bot_dump}))
 
 
 @bots.command(name="update", section="Configuration Commands")
@@ -1126,7 +1238,7 @@ def bot_info(cluster: "ClusterClient", bot_name: str):
     help="registry credentials to use to pull the image",
 )
 @click.argument("name", metavar="BOT")
-@cluster_client
+@cluster_client()
 def update_bot(
     cluster: "ClusterClient",
     new_name: str | None,
@@ -1184,17 +1296,25 @@ def update_bot(
 
     if clear_environment or (environment and bot.environment != list(environment)):
         variable_groups = cluster.variable_groups
-        env = {
-            "old": {
-                vg.name: vg.variables for vg in map(variable_groups.__getitem__, bot.environment)
-            }
-        }
+        env: dict[str, dict[str, list[str]]] = dict(old={}, new={})
+
+        for vg_name in bot.environment:
+            if vg := variable_groups.get(vg_name):
+                env["old"][vg_name] = vg.variables
+
+            else:
+                click.secho(f"Variable Group missing: '{vg_name}'", bold=True, fg="red")
+
         if not clear_environment:
-            env["new"] = {
-                vg.name: vg.variables for vg in map(variable_groups.__getitem__, environment)
-            }
-        else:
-            env["new"] = {}
+            for vg_name in environment:
+                if vg := variable_groups.get(vg_name):
+                    env["new"][vg_name] = vg.variables
+
+                else:
+                    raise click.BadOptionUsage(
+                        "environment", f"Variable Group doesn't exist: '{vg_name}'"
+                    )
+
         click.echo(yaml.safe_dump({"Environment": env}))
 
     redeploy_required |= clear_environment
@@ -1213,7 +1333,7 @@ def update_bot(
         network=network,
         provider=provider,
         account=account,
-        environment=environment if clear_environment else None,
+        environment=environment if environment or clear_environment else None,
         credential_name=credential_name,
     )
 
@@ -1237,67 +1357,66 @@ def update_bot(
     click.echo(yaml.safe_dump(bot_dump))
     if bot.environment:
         click.echo("environment:")
-        click.echo(yaml.safe_dump([var.model_dump() for var in bot.vargroups]))
+        click.echo(
+            yaml.safe_dump([var.model_dump(exclude={"id", "created"}) for var in bot.vargroups])
+        )
 
 
 @bots.command(name="remove", section="Configuration Commands")
-@click.argument("name", metavar="BOT")
-@cluster_client
+@click.argument("name", metavar="BOT", default="*")
+@cluster_client()
 def remove_bot(cluster: "ClusterClient", name: str):
-    """Remove BOT from CLUSTER (Shutdown if running)"""
+    """Remove one or more BOT(s) from CLUSTER (Shutdown if running)"""
 
-    if not (bot := cluster.bots.get(name)):
-        raise click.UsageError(f"Unknown bot '{name}'.")
-
-    elif not click.confirm(f"Do you want to shutdown and delete '{name}'?"):
-        return
-
-    bot.remove()
-    click.secho(f"Bot '{bot.name}' removed.", fg="green", bold=True)
+    for bot in parse_globbed_arg(name, cluster.bots):
+        if not click.confirm(f"Do you want to shutdown and delete '{bot.name}'?"):
+            bot.remove()
+            click.secho(f"Bot '{bot.name}' removed.", fg="green", bold=True)
 
 
 @bots.command(name="health", section="Bot Operation Commands")
-@click.argument("bot_name", metavar="BOT")
-@cluster_client
-def bot_health(cluster: "ClusterClient", bot_name: str):
-    """Show current health of BOT in a CLUSTER"""
+@click.argument("name", metavar="BOT", default="*")
+@cluster_client()
+def bot_health(cluster: "ClusterClient", name: str):
+    """Show current health of one or more BOT(s) in a CLUSTER"""
 
-    if not (bot := cluster.bots.get(bot_name)):
-        raise click.UsageError(f"Unknown bot '{bot_name}'.")
+    for bot in parse_globbed_arg(name, cluster.bots):
+        if bot.is_healthy:
+            bot_status = click.style("healthy", fg="green")
+        else:
+            bot_status = click.style("not healthy", fg="red")
 
-    click.echo("Bot is healthy" if bot.is_healthy else "Bot is not healthy")
+        click.echo(f"{bot.name}: {bot_status}")
 
 
 @bots.command(name="start", section="Bot Operation Commands")
-@click.argument("name", metavar="BOT")
-@cluster_client
+@click.argument("name", metavar="BOT", default="*")
+@cluster_client()
 def start_bot(cluster: "ClusterClient", name: str):
-    """Start BOT running in CLUSTER (if stopped or terminated)"""
+    """Start one or more BOT(s) running in CLUSTER (if stopped or terminated)"""
 
-    if not (bot := cluster.bots.get(name)):
-        raise click.UsageError(f"Unknown bot '{name}'.")
-
-    elif not click.confirm(f"Do you want to start running '{name}'?"):
-        return
-
-    bot.start()
-    click.secho(f"Bot '{bot.name}' starting...", fg="green", bold=True)
+    for bot in parse_globbed_arg(name, cluster.bots):
+        if click.confirm(f"Do you want to start running '{bot.name}'?"):
+            try:
+                bot.start()
+                click.secho(f"Bot '{bot.name}' starting...", fg="green", bold=True)
+            except ClientError as e:
+                click.secho(f"Error starting '{bot.name}': {e}", fg="red")
 
 
 @bots.command(name="stop", section="Bot Operation Commands")
-@click.argument("name", metavar="BOT")
-@cluster_client
+@click.argument("name", metavar="BOT", default="*")
+@cluster_client()
 def stop_bot(cluster: "ClusterClient", name: str):
-    """Stop BOT from running in CLUSTER (if running)"""
+    """Stop one or more BOT(s) from running in CLUSTER (if running)"""
 
-    if not (bot := cluster.bots.get(name)):
-        raise click.UsageError(f"Unknown bot '{name}'.")
-
-    elif not click.confirm(f"Do you want to stop '{name}' from running?"):
-        return
-
-    bot.stop()
-    click.secho(f"Bot '{bot.name}' stopping...", fg="green", bold=True)
+    for bot in parse_globbed_arg(name, cluster.bots):
+        if not click.confirm(f"Do you want to stop '{bot.name}' from running?"):
+            try:
+                bot.stop()
+                click.secho(f"Bot '{bot.name}' stopping...", fg="green", bold=True)
+            except ClientError as e:
+                click.secho(f"Error stopping '{bot.name}': {e}", fg="red")
 
 
 @bots.command(name="logs", section="Bot Operation Commands")
@@ -1316,8 +1435,21 @@ def stop_bot(cluster: "ClusterClient", name: str):
     help="Return logs since N ago.",
     callback=timedelta_callback,
 )
-@cluster_client
-def show_bot_logs(cluster: "ClusterClient", name: str, log_level: str, since: timedelta | None):
+@click.option(
+    "-f",
+    "--follow",
+    help="Stream logs as they come in",
+    is_flag=True,
+    default=False,
+)
+@cluster_client()
+def show_bot_logs(
+    cluster: "ClusterClient",
+    name: str,
+    log_level: str,
+    since: timedelta | None,
+    follow: bool,
+):
     """Show runtime logs for BOT in CLUSTER"""
 
     start_time = None
@@ -1332,18 +1464,34 @@ def show_bot_logs(cluster: "ClusterClient", name: str, log_level: str, since: ti
     except KeyError:
         level = LogLevel.INFO
 
-    for log in bot.filter_logs(log_level=level, start_time=start_time):
-        click.echo(log)
+    for log in bot.get_logs(log_level=level, start_time=start_time, follow=follow):
+        click.echo(str(log))
 
 
 @bots.command(name="errors", section="Bot Operation Commands")
-@click.argument("name", metavar="BOT")
-@cluster_client
+@click.argument("name", metavar="BOT", default="*")
+@cluster_client()
 def show_bot_errors(cluster: "ClusterClient", name: str):
-    """Show unacknowledged errors for BOT in CLUSTER"""
+    """Show unacknowledged errors for one or more BOT(s) in CLUSTER"""
 
-    if not (bot := cluster.bots.get(name)):
-        raise click.UsageError(f"Unknown bot '{name}'.")
+    for bot in parse_globbed_arg(name, cluster.bots):
+        click.echo(f"'{bot.name}' errors:")
+        for log in bot.errors:
+            click.echo(log)
 
-    for log in bot.errors:
-        click.echo(log)
+
+@cluster.command(name="mcp", section="Platform Commands (https://silverback.apeworx.io)")
+@cluster_client(show_login=False)
+def run_mcp_server(cluster: "ClusterClient"):
+    """Run MCP (Model Context Protocol) Server"""
+
+    try:
+        from .cluster import mcp
+    except ImportError:
+        raise click.UsageError(
+            "You must install the `mcp` package (or use `silverback[mcp]` extra)"
+        )
+
+    # NOTE: Need to inject this into context so it has access
+    mcp.client = cluster
+    mcp.server.run()
