@@ -1,23 +1,30 @@
 import atexit
 import inspect
 from collections import defaultdict
-from datetime import timedelta
+from datetime import datetime, timedelta
 from functools import wraps
+from types import MethodType
 from typing import Any, Awaitable, Callable
 
+import pycron  # type: ignore[import-untyped]
 from ape.api.networks import LOCAL_NETWORK_NAME
-from ape.contracts import ContractEvent, ContractInstance
+from ape.contracts import ContractEvent, ContractEventWrapper, ContractInstance
 from ape.logging import logger
 from ape.managers.chain import BlockContainer
+from ape.types import AddressType, ContractLog
 from ape.utils import ManagerAccessMixin
+from eth_typing import HexStr
+from eth_utils import keccak, to_hex
+from ethpm_types.abi import encode_topic_value
 from packaging.version import Version
 from pydantic import BaseModel
 from taskiq import AsyncTaskiqDecoratedTask, TaskiqEvents
 
-from .exceptions import ContainerTypeMismatchError, InvalidContainerTypeError
+from .exceptions import ContainerTypeMismatchError, InvalidContainerTypeError, NoSignerLoaded
 from .settings import Settings
 from .state import StateSnapshot
-from .types import SilverbackID, TaskType
+from .types import ScalarType, SilverbackID, TaskType
+from .utils import encode_topics_to_string, parse_hexbytes_dict
 
 
 class SystemConfig(BaseModel):
@@ -32,7 +39,7 @@ class SystemConfig(BaseModel):
 class TaskData(BaseModel):
     # NOTE: Data we need to know how to call a task via kicker
     name: str  # Name of user function
-    labels: dict[str, Any]
+    labels: dict[str, str]
 
     # NOTE: Any other items here must have a default value
 
@@ -125,7 +132,9 @@ class SilverbackBot(ManagerAccessMixin):
         ):
             settings.NEW_BLOCK_TIMEOUT = int(timedelta(days=1).total_seconds())
 
-        settings_str = "\n  ".join(f'{key}="{val}"' for key, val in settings.dict().items() if val)
+        settings_str = "\n  ".join(
+            f'{key}="{val}"' for key, val in settings.model_dump().items() if val
+        )
         logger.info(f"Loading Silverback Bot with settings:\n  {settings_str}")
 
         self.broker = settings.get_broker()
@@ -140,6 +149,16 @@ class SilverbackBot(ManagerAccessMixin):
         atexit.register(provider_context.__exit__, None, None, None)
 
         self.signer = settings.get_signer()
+        if self.signer:
+            # NOTE: Monkeypatch `AccountAPI.call` to update bot nonce tracking state
+            original_call = self.signer.call
+
+            def call_override(account_instance, txn, *args, **kwargs):
+                self.state["system:last_nonce_used"] = txn.nonce
+                return original_call(txn, *args, **kwargs)
+
+            self.signer.__dict__["call"] = MethodType(call_override, self.signer)
+
         self.new_block_timeout = settings.NEW_BLOCK_TIMEOUT
         self.use_fork = settings.FORK_MODE and not self.provider.network.name.endswith("-fork")
 
@@ -178,9 +197,21 @@ class SilverbackBot(ManagerAccessMixin):
         self, task_type: TaskType, task_handler: Callable
     ) -> AsyncTaskiqDecoratedTask:
         assert str(task_type).startswith("system:"), "Can only add system tasks"
+
+        # NOTE: We need this as `.register_task` tries to update `.__name__` of `task_handler`,
+        #       but methods do not allow setting this attribute (raises AttributeError)
+        @wraps(task_handler)
+        async def call_task_handler(*args, **kwargs):
+            result = task_handler(*args, **kwargs)
+
+            if inspect.isawaitable(result):
+                return await result
+
+            return result
+
         # NOTE: This has to be registered with the broker in the worker
         return self.broker.register_task(
-            task_handler,
+            call_task_handler,
             # NOTE: Name makes it impossible to conflict with user's handler fn names
             task_name=str(task_type),
             task_type=str(task_type),
@@ -211,9 +242,37 @@ class SilverbackBot(ManagerAccessMixin):
 
         self.state["system:last_block_seen"] = startup_state.last_block_seen
         self.state["system:last_block_processed"] = startup_state.last_block_processed
+
+        if self.signer:
+            # NOTE: 'BaseAddress.nonce` is 1 + last nonce that was used
+            self.state["system:last_nonce_used"] = max(
+                startup_state.last_nonce_used or -1, self.signer.nonce - 1
+            )
+
         # TODO: Load user custom state (should not start with `system:`)
 
-    async def __create_snapshot_handler(
+    async def __create_snapshot_handler(self) -> StateSnapshot:
+        return StateSnapshot(
+            # TODO: Migrate these to parameters (remove explicitly from state)
+            last_block_seen=self.state.get("system:last_block_seen", -1),
+            last_block_processed=self.state.get("system:last_block_processed", -1),
+            last_nonce_used=self.state.get("system:last_nonce_used"),
+        )
+
+    @property
+    def nonce(self) -> int:
+        if not self.signer:
+            raise NoSignerLoaded()
+
+        elif (last_nonce_used := self.state.get("system:last_nonce_used")) is None:
+            raise AttributeError(
+                "`bot.state` not fully loaded yet, please do not use during worker startup."
+            )
+
+        # NOTE: Next nonce (`.nonce` is meant to be used in next txn) is 1 + last
+        return max(last_nonce_used + 1, self.signer.nonce)
+
+    def _checkpoint(
         self,
         last_block_seen: int | None = None,
         last_block_processed: int | None = None,
@@ -225,11 +284,54 @@ class SilverbackBot(ManagerAccessMixin):
         if last_block_processed is not None:
             self.state["system:last_block_processed"] = last_block_processed
 
-        return StateSnapshot(
-            # TODO: Migrate these to parameters (remove explicitly from state)
-            last_block_seen=self.state.get("system:last_block_seen", -1),
-            last_block_processed=self.state.get("system:last_block_processed", -1),
-        )
+    def _ensure_block(self, handler: Callable) -> Callable:
+        @wraps(handler)
+        async def ensure_block(block, *args, **kwargs):
+            # NOTE: With certain runners, `block` may be raw or in it's final form
+            if isinstance(block, dict):
+                block = self.provider.network.ecosystem.decode_block(parse_hexbytes_dict(block))
+
+            self._checkpoint(last_block_seen=block.number)
+            result = handler(block, *args, **kwargs)
+            self._checkpoint(last_block_processed=block.number)
+
+            if inspect.isawaitable(result):
+                return await result
+
+            return result
+
+        return ensure_block
+
+    def _ensure_log(self, event: ContractEvent, handler: Callable) -> Callable:
+        @wraps(handler)
+        async def ensure_log(log, *args, **kwargs):
+            # NOTE: With certain runners, `log` may be raw or in it's final form
+            if isinstance(log, dict):
+                if "event_arguments" in log:
+                    # This is an Ape object, simply initialize it
+                    log = ContractLog(**log)
+
+                else:  # This is a raw web3py object
+                    log = next(  # NOTE: `next` is okay since it only has one item
+                        self.provider.network.ecosystem.decode_logs(
+                            [parse_hexbytes_dict(log)], event.abi
+                        )
+                    )
+                    # TODO: Fix upstream w/ web3py
+                    log.transaction_hash = "0x" + log.transaction_hash.hex()
+
+            self._checkpoint(last_block_seen=log.block.number)
+            result = handler(log, *args, **kwargs)
+            self._checkpoint(last_block_processed=log.block.number)
+
+            if inspect.isawaitable(result):
+                return await result
+
+            return result
+
+        # NOTE: Avoid processing w/ TaskIQ's automatic `BaseModel` parser
+        ensure_log.__annotations__["log"] = dict
+        return ensure_log
 
     # To ensure we don't have too many forks at once
     # HACK: Until `NetworkManager.fork` (and `ProviderContextManager`) allows concurrency
@@ -250,10 +352,35 @@ class SilverbackBot(ManagerAccessMixin):
 
         return fork_handler
 
+    def _convert_arg_to_hexstr(self, arg_value: Any, arg_type: str) -> HexStr | list[HexStr] | None:
+        python_type: Any
+        if "int" in arg_type:
+            python_type = int
+        elif "bytes" in arg_type:
+            python_type = bytes
+        elif arg_type == "address":
+            python_type = AddressType
+        elif arg_type == "string":
+            python_type = str
+        else:
+            raise ValueError(f"Unable to support ABI Type '{arg_type}'.")
+
+        if isinstance(arg_value, list):
+            arg_value = [self.conversion_manager.convert(v, python_type) for v in arg_value]
+
+        else:
+            arg_value = self.conversion_manager.convert(arg_value, python_type)
+
+        return encode_topic_value(arg_type, arg_value)  # type: ignore[return-value]
+
     def broker_task_decorator(
         self,
         task_type: TaskType,
-        container: BlockContainer | ContractEvent | None = None,
+        container: BlockContainer | ContractEvent | ContractEventWrapper | None = None,
+        filter_args: dict[str, Any] | None = None,
+        cron_schedule: str | None = None,
+        metric_name: str | None = None,
+        value_threshold: dict[str, ScalarType] | None = None,
     ) -> Callable[[Callable], AsyncTaskiqDecoratedTask]:
         """
         Dynamically create a new broker task that handles tasks of ``task_type``.
@@ -278,7 +405,10 @@ class SilverbackBot(ManagerAccessMixin):
         """
         if (
             (task_type is TaskType.NEW_BLOCK and not isinstance(container, BlockContainer))
-            or (task_type is TaskType.EVENT_LOG and not isinstance(container, ContractEvent))
+            or (
+                task_type is TaskType.EVENT_LOG
+                and not isinstance(container, (ContractEvent, ContractEventWrapper))
+            )
             or (
                 task_type
                 not in (
@@ -290,24 +420,80 @@ class SilverbackBot(ManagerAccessMixin):
         ):
             raise ContainerTypeMismatchError(task_type, container)
 
+        elif isinstance(container, ContractEventWrapper):
+            if len(container.events) != 1:
+                raise InvalidContainerTypeError(
+                    f"Requires exactly 1 event to unwrap: {container.events}"
+                )
+            container = container.events[0]
+
         # Register user function as task handler with our broker
         def add_taskiq_task(
             handler: Callable[..., Any | Awaitable[Any]]
         ) -> AsyncTaskiqDecoratedTask:
-            labels = {"task_type": str(task_type)}
+            labels: dict[str, str] = dict()
 
-            # NOTE: Do *not* do `if container` because that does a `len(container)` call,
-            #       which for ContractEvent queries *every single log* ever emitted, and really
-            #       we only want to determine if it is not None
-            if container is not None and isinstance(container, ContractEvent):
-                # Address is almost a certainty if the container is being used as a filter here.
-                if not (contract_address := getattr(container.contract, "address", None)):
+            if task_type is TaskType.NEW_BLOCK:
+                handler = self._ensure_block(handler)
+
+            elif task_type is TaskType.EVENT_LOG:
+                assert container is not None and isinstance(container, ContractEvent)
+                # NOTE: allows broad capture filters (matching multiple addresses)
+                if (contract := getattr(container, "contract", None)) and hasattr(
+                    contract, "address"
+                ):
+                    labels["address"] = contract.address
+
+                labels["event"] = container.abi.signature
+
+                topics: list[list[HexStr] | HexStr | None] = [
+                    # Topic 0: event_id
+                    to_hex(keccak(text=container.abi.selector))
+                ]
+
+                # Topic 1-3: event args ([..., ...] represent OR)
+                if filter_args:
+                    for arg in container.abi.inputs:
+                        if not arg.indexed:
+                            break  # Inputs should be ordered indexed first
+
+                        if arg_value := filter_args.pop(arg.name, None):
+                            topics.append(self._convert_arg_to_hexstr(arg_value, arg.type))
+
+                        else:
+                            # Skip this indexed argument (`None` is wildcard match)
+                            topics.append(None)
+                            # NOTE: Will clean up extra Nones in `encode_topics_to_string`
+
+                    if unmatched_args := "', '".join(filter_args):
+                        raise InvalidContainerTypeError(
+                            f"Args are not available for filtering: '{unmatched_args}'."
+                        )
+
+                labels["topics"] = encode_topics_to_string(topics)
+
+                handler = self._ensure_log(container, handler)
+
+            elif task_type is TaskType.CRON_JOB:
+                # NOTE: If cron schedule has never been true over a year timeframe, it's bad
+                if not cron_schedule or not pycron.has_been(
+                    cron_schedule, datetime.now() - timedelta(days=366)
+                ):
                     raise InvalidContainerTypeError(
-                        "Please provide a contract event from a valid contract instance."
+                        f"'{cron_schedule}' is not a valid cron schedule"
                     )
 
-                labels["contract_address"] = contract_address
-                labels["event_signature"] = container.abi.signature
+                labels["cron"] = cron_schedule
+
+            elif task_type is TaskType.METRIC_VALUE:
+                # NOTE: This shouldn't happen to users
+                assert metric_name, "Must supply `metric_name=`."
+                labels["metric"] = metric_name
+
+                if value_threshold:
+                    labels.update(
+                        {f"value:{lbl}": str(val) for lbl, val in value_threshold.items()}
+                    )
 
             self.tasks[task_type].append(TaskData(name=handler.__name__, labels=labels))
 
@@ -317,6 +503,7 @@ class SilverbackBot(ManagerAccessMixin):
             return self.broker.register_task(
                 handler,
                 task_name=handler.__name__,
+                task_type=str(task_type),
                 **labels,
             )
 
@@ -390,6 +577,8 @@ class SilverbackBot(ManagerAccessMixin):
         # TODO: possibly remove these
         new_block_timeout: int | None = None,
         start_block: int | None = None,
+        filter_args: dict[str, Any] | None = None,
+        **filter_kwargs: dict[str, Any],
     ):
         """
         Create task to handle events created by the `container` trigger.
@@ -420,25 +609,91 @@ class SilverbackBot(ManagerAccessMixin):
 
             return self.broker_task_decorator(TaskType.NEW_BLOCK, container=container)
 
-        elif isinstance(container, ContractEvent) and isinstance(
-            container.contract, ContractInstance
-        ):
-            key = container.contract.address
+        elif isinstance(container, ContractEvent):
+            if isinstance(container.contract, ContractInstance):
+                key = container.contract.address
+                if new_block_timeout is not None:
+                    if key in self.poll_settings:
+                        self.poll_settings[key]["new_block_timeout"] = new_block_timeout
+                    else:
+                        self.poll_settings[key] = {"new_block_timeout": new_block_timeout}
 
-            if new_block_timeout is not None:
-                if key in self.poll_settings:
-                    self.poll_settings[key]["new_block_timeout"] = new_block_timeout
-                else:
-                    self.poll_settings[key] = {"new_block_timeout": new_block_timeout}
+                if start_block is not None:
+                    if key in self.poll_settings:
+                        self.poll_settings[key]["start_block"] = start_block
+                    else:
+                        self.poll_settings[key] = {"start_block": start_block}
 
-            if start_block is not None:
-                if key in self.poll_settings:
-                    self.poll_settings[key]["start_block"] = start_block
-                else:
-                    self.poll_settings[key] = {"start_block": start_block}
+            if filter_args:
+                filter_kwargs.update(filter_args)
 
-            return self.broker_task_decorator(TaskType.EVENT_LOG, container=container)
+            return self.broker_task_decorator(
+                TaskType.EVENT_LOG,
+                container=container,
+                filter_args=filter_kwargs,
+            )
 
         # TODO: Support account transaction polling
         # TODO: Support mempool polling?
         raise InvalidContainerTypeError(container)
+
+    def cron(self, cron_schedule: str) -> Callable:
+        """
+        Create task to run on a schedule.
+
+        Args:
+            cron_schedule (str): A cron-like schedule string.
+        """
+        return self.broker_task_decorator(TaskType.CRON_JOB, cron_schedule=cron_schedule)
+
+    def on_metric(
+        self,
+        metric_name: str,
+        ge: ScalarType | None = None,
+        gt: ScalarType | None = None,
+        le: ScalarType | None = None,
+        lt: ScalarType | None = None,
+        eq: ScalarType | None = None,
+        ne: ScalarType | None = None,
+        # TODO: Support `rate_[ge|gt|le|...]` too?
+    ) -> Callable:
+        """
+        Create a task that runs when the value of a specified metric has tripped a threshold.
+
+        ```{notice}
+        If no keyword args provided to this decorator, it will trigger on every update of metric.
+        ```
+
+        ```{notice}
+        Multiple keyword args provided to this decorator means logical AND of all of them.
+        ```
+
+        Args:
+            metric_name (str): The name of the metric to monitor for threshold exceedence.
+            ge (ScalarType | None): trigger when metric value is greater than or equal to value.
+            gt (ScalarType | None): trigger when metric value is greater than value.
+            le (ScalarType | None): trigger when metric value is less than or equal to value.
+            lt (ScalarType | None): trigger when metric value is less than value.
+            eq (ScalarType | None): trigger when metric value is equal to value.
+            ne (ScalarType | None): trigger when metric value is not equal to value.
+        """
+        value_threshold: dict[str, ScalarType] = {}
+        if ge is not None:
+            value_threshold["ge"] = ge
+        if gt is not None:
+            value_threshold["gt"] = gt
+        if le is not None:
+            value_threshold["le"] = le
+        if lt is not None:
+            value_threshold["lt"] = lt
+        if eq is not None:
+            value_threshold["eq"] = eq
+        if ne is not None:
+            value_threshold["ne"] = ne
+
+        return self.broker_task_decorator(
+            TaskType.METRIC_VALUE,
+            metric_name=metric_name,
+            # NOTE: When empty, allow all values
+            value_threshold=value_threshold or None,
+        )
